//http://beej.us/blog/data/javascript-gamepad/

var GamepadKey = {
    A :         0,
    B :         1,
    X :         2,
    Y :         3,
    L1 :        4,
    R1 :        5,
    L2 :        6,
    R2 :        7,
    VIEW :      8,
    OPTION :    9,
    L3 :        10,
    R3 :        11,
    UP :        12,
    DOWN:       13,
    LEFT :      14,
    RIGHT:      15
}

/*
gamepad.axes[1]:
   -1: left
   +1: right

gamepad.axes[2]:
   -1: up
   +1: down
*/

var Gamepad = (function () {
	
	var oldPad = [];
	var leftDX = 0;
	var leftDY = 0;
	var leftFX = 0;
	var leftFY = 0;	
	var lastLeftMoveFX = 0;
	var lastLeftMoveFY = 0;	

    var oldTime = 0;
    var deltaTime = 200; 
	
	var thresholdValue = 0.25;

    Math.fmod = function (a,b) 
	{ 
	    return Number((a - (Math.floor(a / b) * b)).toPrecision(8)); 
	};
	
	var getAngle = function(x, y)
	{
		var INV_PI = 180 / 3.141592;
		var degree = Math.atan2(y, x) * INV_PI;
		//degree = Math.fmod(degree + 360, 360);
		return degree;
	};
	
    function preUpdate() {
        var pad = navigator.getGamepads()[0];
		
        var currTime = Date.now();

		if ((currTime - oldTime) >= deltaTime) {
							
			if (Math.abs(pad.axes[0]) >= thresholdValue || Math.abs(pad.axes[1]) >= thresholdValue) {
				leftDX = Math.round(pad.axes[0]);
				leftDY = Math.round(pad.axes[1]);
				oldTime = Date.now();
				
				if((pad.axes[0]*pad.axes[0] + pad.axes[1]*pad.axes[1]) >= 1 )
				{
					leftFX = lastLeftMoveFX = applyDeadzone(pad.axes[0], thresholdValue);
					leftFY = lastLeftMoveFY = applyDeadzone(pad.axes[1], thresholdValue);
				}
			}
			
			
		}

		/*
		if (pad.buttons[0].pressed) 
			console.log('A Button');
			*/
	};		

	var applyDeadzone = function(number, threshold){
	   percentage = (Math.abs(number) - threshold) / (1 - threshold);

	   if(percentage < 0)
		  percentage = 0;

	   return percentage * (number > 0 ? 1 : -1);
	}	
	
	
	function postUpdate() {
		var pad = navigator.getGamepads()[0];
		
		for (n = 0; n < oldPad.length; ++n) 
			oldPad[n] = pad.buttons[n].pressed;
		
		leftDX = 0;
		leftDY = 0;
		leftFX = 0;
		leftFY = 0;		
	}
			
	function getLeftStick()
	{
		return {
			x: leftDX,
			y: leftDY,
			fx: leftFX,
			fy: leftFY
		};
	}	
	
	function getLastLeftAngle()
	{
		return getAngle(lastLeftMoveFX, lastLeftMoveFY);
	}
	
	//first pressed button
	function click(key)
	{
		var pad = navigator.getGamepads()[0];
		return (!oldPad[key] && pad.buttons[key].pressed);
	}
	
	function moveClick()
	{
		var pad = navigator.getGamepads()[0];
		
		var x = 0;
		var y = 0;
		if(!oldPad[GamepadKey.LEFT] && pad.buttons[GamepadKey.LEFT].pressed)
			x += -1;
		if(!oldPad[GamepadKey.RIGHT] && pad.buttons[GamepadKey.RIGHT].pressed)
			x += 1;
		if(!oldPad[GamepadKey.UP] && pad.buttons[GamepadKey.UP].pressed)
			y = -1;
		if(!oldPad[GamepadKey.DOWN] && pad.buttons[GamepadKey.DOWN].pressed)
			y = 1;
		if(leftDX != 0)
			x = leftDX;
		if(leftDY != 0)
			y = leftDY;
		
		return {
			x: x,
			y: y
		};
	}
	
	function init()
	{
		var pad = navigator.getGamepads()[0];
		if (oldPad.length == 0)
		{
			oldPad.length = pad.buttons.length;
			for (n = 0; n < oldPad.length; ++n)
			{
				oldPad[n] = false;
			}
		}		
	}
	
			
    return {
		init: init,
		preUpdate: preUpdate,
		postUpdate: postUpdate,
		click: click,
		getLeftStick: getLeftStick,
		getLastLeftAngle: getLastLeftAngle,
		getAngle: getAngle,
		moveClick: moveClick
    };	
})();